package fileserver.client;

import fileserver.share.*;
import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import javax.swing.tree.*;
import javax.swing.event.*;
import java.util.*;
import java.rmi.RemoteException;
import java.io.File;

/**
 * <p>Title: File Server</p>
 * <p>Description: Bezestavov File Server pomoc RMI</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: </p>
 * @author Jan Doleel
 * @version 1.0
 */

public class MainFrame extends JFrame implements TreeSelectionListener {
  private FileServerInt fs;
  private Session sess;
  private TreePath last = null;

  DefaultMutableTreeNode top;
  JPanel jPanel1 = new JPanel();
  GridLayout gridLayout2 = new GridLayout();
  JScrollPane jScrollPane1 = new JScrollPane();
  JScrollPane jScrollPane2 = new JScrollPane();
  private JTree tree;
  private DefaultTreeModel model;
  JList list = new JList();
  BorderLayout borderLayout1 = new BorderLayout();
  JToolBar jToolBar1 = new JToolBar();
  JButton reload = new JButton();
  JButton put = new JButton();
  JButton get = new JButton();
  JButton share = new JButton();
  JButton delete = new JButton();
  JButton newShare = new JButton();
  JButton Create = new JButton();

  public MainFrame(FileServerInt f, Session s) {
    fs = f;
    sess = s;
    try {
      jbInit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
    list.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    this.reload_actionPerformed(null);
  }

  private void jbInit() throws Exception {
    this.setTitle("FileServer - client");
    this.setSize(new Dimension(600, 400));
    this.getContentPane().setLayout(borderLayout1);
    this.addWindowListener(new MainFrame_this_windowAdapter(this));
    jPanel1.setLayout(gridLayout2);
    reload.setText("Reload");
    reload.addActionListener(new MainFrame_reload_actionAdapter(this));
    put.setText("Put");
    put.addActionListener(new MainFrame_put_actionAdapter(this));
    get.setText("Get");
    get.addActionListener(new MainFrame_get_actionAdapter(this));
    share.setText("Share");
    share.addActionListener(new MainFrame_share_actionAdapter(this));
    delete.setText("Delete");
    delete.addActionListener(new MainFrame_delete_actionAdapter(this));
    newShare.setText("Share new...");
    newShare.addActionListener(new MainFrame_newShare_actionAdapter(this));
    Create.setText("Create Dir");
    Create.addActionListener(new MainFrame_Create_actionAdapter(this));
    this.getContentPane().add(jPanel1, BorderLayout.CENTER);
    jPanel1.add(jScrollPane1, null);
    jPanel1.add(jScrollPane2, null);
    this.getContentPane().add(jToolBar1, BorderLayout.NORTH);
    jToolBar1.add(reload, null);
    jScrollPane2.getViewport().add(list, null);
    jToolBar1.add(put, null);
    jToolBar1.add(get, null);
    jToolBar1.add(delete, null);
    jToolBar1.add(Create, null);
    jToolBar1.add(share, null);
    jToolBar1.add(newShare, null);
  }

  void this_windowClosing(WindowEvent e) {
    System.exit(1);
  }

  private void createNodes(DefaultMutableTreeNode top) {
    top.removeAllChildren();
    try {
      FileList l = fs.list(sess);
      if(l != null) {
        l.sort();
        Iterator i = l.iterator();
        Vector v = new Vector(10);
        while (i.hasNext()) {
          FileNode node = (FileNode) i.next();
          if (node.isDir()) {
            top.add(node);
          }
          else {
            v.add(node);
          }
          list.setListData(v);
        }
      }
    }
    catch(RemoteException e) {
      System.out.println("Remote exception");
      e.printStackTrace();
      System.exit(1);
    }
    if(model != null) {
      model.reload(top);
    }
  }

  public void valueChanged(TreeSelectionEvent e) {
//    System.out.println("changed");
    last = e.getPath();
    this.update(e.getNewLeadSelectionPath());
  }

  private void update(TreePath n) {
    if(n == null) {
      return;
    }
    if(top.equals(n.getLastPathComponent())) {
      createNodes(top);
    }
    else {
      FileNode node = (FileNode) n.getLastPathComponent();
      if(node != null) {
        sess.setFile(new File(node.getFullPath()));
        createNodes(node);
      }
    }
    tree.expandPath(n);
  }

  void share_actionPerformed(ActionEvent e) {
    if((tree.getSelectionPath() != null) && !tree.getSelectionPath().getLastPathComponent().equals(top)) {
      sess.setFile(new File(((FileNode) tree.getSelectionPath().getLastPathComponent()).getFullPath()));
      try {
        fs.setShare(sess);
      }
      catch(RemoteException ex) {
        System.out.println("Remote exception");
        ex.printStackTrace();
        System.exit(1);
      }
      this.reload_actionPerformed(null);
    }
  }

  void reload_actionPerformed(ActionEvent e) {
//    System.out.println("reload");
    top = new DefaultMutableTreeNode("Shared Root");
    sess.setFile(null);
    this.createNodes(top);
    model = new DefaultTreeModel(top);
    tree = new JTree(model);
    DefaultTreeCellRenderer renderer = new DefaultTreeCellRenderer();
    renderer.setLeafIcon(renderer.getClosedIcon());
    tree.setCellRenderer(renderer);
    tree.addTreeSelectionListener(this);
    tree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
    jScrollPane1.getViewport().add(tree, null);
    tree.setRootVisible(false);
  }

  void put_actionPerformed(ActionEvent e) {
    if((tree.getSelectionPath() != null) && !tree.getSelectionPath().getLastPathComponent().equals(top)) {
      JFileChooser chooser = new JFileChooser();
      if (JFileChooser.APPROVE_OPTION == chooser.showOpenDialog(this)) {
        sess.setFile(new File (((FileNode) tree.getSelectionPath().getLastPathComponent()).getFullPath()));
        try {
          MyFile f = new MyFile(chooser.getSelectedFile().getPath());
          if(f != null) {
            fs.put(f, sess);
          }
        }
        catch(RemoteException ex) {
          System.out.println("Remote exception");
          ex.printStackTrace();
        }
        this.update(last);
      }
    }
  }

  void get_actionPerformed(ActionEvent e) {
    if(list.getSelectedValue() != null) {
      JFileChooser chooser = new JFileChooser();
      if (JFileChooser.APPROVE_OPTION == chooser.showSaveDialog(this)) {
        sess.setFile(new File(((FileNode) list.getSelectedValue()).getFullPath()));
        try {
          MyFile f = fs.get(sess);
          if(f != null) {
            File dir = new File(chooser.getSelectedFile().getParent());
            if(dir != null) {
              f.saveFile(dir);
            }
          }
        }
        catch(RemoteException ex) {
          System.out.println("Remote exception");
          ex.printStackTrace();
        }
      }
    }
  }

  void delete_actionPerformed(ActionEvent e) {
    if(list.getSelectedValue() != null) {
      sess.setFile(new File(((FileNode) list.getSelectedValue()).getFullPath()));
      try {
        fs.delete(sess);
      }
      catch(RemoteException ex) {
        System.out.println("Remote exception");
        ex.printStackTrace();
      }
      this.update(last);
    }
    else if(tree.getSelectionPath() != null) {
      sess.setFile(new File(((FileNode) tree.getSelectionPath().getLastPathComponent()).getFullPath()));
      try {
        fs.delete(sess);
      }
      catch(RemoteException ex) {
        System.out.println("Remote exception");
        ex.printStackTrace();
      }
      this.update(last);
    }
  }

  void newShare_actionPerformed(ActionEvent e) {
    JFileChooser chooser = new JFileChooser();
    if (JFileChooser.APPROVE_OPTION == chooser.showOpenDialog(this)) {
      sess.setFile(chooser.getSelectedFile().getParentFile());
      try {
        fs.setShare(sess);
      }
      catch(RemoteException ex) {
        System.out.println("Remote exception");
        ex.printStackTrace();
      }
      this.reload_actionPerformed(null);
    }
  }

  void Create_actionPerformed(ActionEvent e) {
    if(tree.getSelectionPath() != null) {
      final JFrame d = new JFrame("Jmeno adresare:");
      final JTextField dname = new JTextField(20);
      JButton b = new JButton("OK");

      class l implements ActionListener {
        public void actionPerformed(ActionEvent e) {
          if(!dname.getText().equals("")) {
            sess.setFile(new File( ( (FileNode) tree.getSelectionPath().
                                    getLastPathComponent()).getFullPath() + "/" +
                                  dname.getText()));
            try {
              fs.createDir(sess);
            }
            catch (RemoteException ex) {
              System.out.println("Remote exception");
              ex.printStackTrace();
            }
          }
          d.setVisible(false);
          update(last);
        }
      }

      b.addActionListener(new l());
      d.getContentPane().setLayout(new BorderLayout());
      d.getContentPane().add(dname, BorderLayout.CENTER);
      d.getContentPane().add(b, BorderLayout.EAST);
      d.setSize(new Dimension(200,60));
      d.setLocation(this.getLocation());
      d.pack();
      d.setVisible(true);
    }
  }
}

class MainFrame_this_windowAdapter extends java.awt.event.WindowAdapter {
  MainFrame adaptee;

  MainFrame_this_windowAdapter(MainFrame adaptee) {
    this.adaptee = adaptee;
  }
  public void windowClosing(WindowEvent e) {
    adaptee.this_windowClosing(e);
  }
}

class MainFrame_share_actionAdapter implements java.awt.event.ActionListener {
  MainFrame adaptee;

  MainFrame_share_actionAdapter(MainFrame adaptee) {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e) {
    adaptee.share_actionPerformed(e);
  }
}

class MainFrame_reload_actionAdapter implements java.awt.event.ActionListener {
  MainFrame adaptee;

  MainFrame_reload_actionAdapter(MainFrame adaptee) {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e) {
    adaptee.reload_actionPerformed(e);
  }
}

class MainFrame_get_actionAdapter implements java.awt.event.ActionListener {
  MainFrame adaptee;

  MainFrame_get_actionAdapter(MainFrame adaptee) {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e) {
    adaptee.get_actionPerformed(e);
  }
}

class MainFrame_put_actionAdapter implements java.awt.event.ActionListener {
  MainFrame adaptee;

  MainFrame_put_actionAdapter(MainFrame adaptee) {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e) {
    adaptee.put_actionPerformed(e);
  }
}

class MainFrame_delete_actionAdapter implements java.awt.event.ActionListener {
  MainFrame adaptee;

  MainFrame_delete_actionAdapter(MainFrame adaptee) {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e) {
    adaptee.delete_actionPerformed(e);
  }
}

class MainFrame_newShare_actionAdapter implements java.awt.event.ActionListener {
  MainFrame adaptee;

  MainFrame_newShare_actionAdapter(MainFrame adaptee) {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e) {
    adaptee.newShare_actionPerformed(e);
  }
}

class MainFrame_Create_actionAdapter implements java.awt.event.ActionListener {
  MainFrame adaptee;

  MainFrame_Create_actionAdapter(MainFrame adaptee) {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e) {
    adaptee.Create_actionPerformed(e);
  }
}